//Bibliotecas para a integrao
#include <jni.h>
#include "NativeHaptic.h"

#include <stdio.h>
#include <conio.h>

//Bibliotecas do OpenHaptics Toolkit
#include <HD/hd.h>
#include <HDU/hdu.h>
#include <HDU/hduError.h>
#include <HDU/hduVector.h>

double KStiffness;//Tenacidade mxima
int supportedCalibrationStyles;
int calibrationStyle;

//Ponteiros para armazenar informaes do dispositivo
double* position = new double(3);
double* velocity = new double(3);
double* transform = new double(16);
double* force = new double(3);

HDCallbackCode HDCALLBACK UpdateCalibrationCallback(int pUserData);
HDCallbackCode HDCALLBACK CalibrationStatusCallback(void *pUserData);

HDSchedulerHandle hMain;
HDErrorInfo error;

//Variveis de verificao de erros e estado dos botes
int currentButton1, lastButton1, currentButton2, lastButton2, button1, button2, verify = 0, verifyThread = 0;

static HHD hHD = HD_INVALID_HANDLE;

//CALLBACK principal
HDCallbackCode HDCALLBACK MainCallback(void *data);

//Obtm a tenacidade mxima proporcionada pelo dispositivo
JNIEXPORT jdouble JNICALL 
Java_ViMeT_NativeHaptic_getStiffness(JNIEnv *env, jobject obj)
{
	hdGetDoublev(HD_NOMINAL_MAX_STIFFNESS,&KStiffness);
	return KStiffness;
}

//Retorna o estado do boto 1
JNIEXPORT jint JNICALL
Java_ViMeT_NativeHaptic_button1State(JNIEnv *env, jobject obj)
{
	return button1;
}

//Retorna o estado do boto 2
JNIEXPORT jint JNICALL
Java_ViMeT_NativeHaptic_button2State(JNIEnv *env, jobject obj)
{
	return button2;
}

//Inicializao do dispositivo hptico, do scheduler
//Verificao de erros
JNIEXPORT jint JNICALL 
Java_ViMeT_NativeHaptic_initializeDevice(JNIEnv *env, jobject obj)
{
	hHD = hdInitDevice(HD_DEFAULT_DEVICE);

	if (HD_DEVICE_ERROR(error = hdGetError()))
	{
		verify = -1;
	}
	else
	{
		hMain = hdScheduleAsynchronous(
		MainCallback, 0, HD_MAX_SCHEDULER_PRIORITY);

		hHD = hdGetCurrentDevice();

		//Habilita o retorno de fora
		hdEnable(HD_FORCE_OUTPUT);

		hdStartScheduler();

		//Erro no Scheduler
		if (HD_DEVICE_ERROR(error = hdGetError()))
		{	
			verify = -2;
		}
		else
		{
				/////////////////////////////////////////////
				//Cdigo para verificar tipo de calibrao a ser executada
				//de acordo com o tipo de dispositivo
				/*hdGetIntegerv(HD_CALIBRATION_STYLE, &supportedCalibrationStyles);
				if (supportedCalibrationStyles & HD_CALIBRATION_ENCODER_RESET)
				{
					calibrationStyle = HD_CALIBRATION_ENCODER_RESET;
				}
				if (supportedCalibrationStyles & HD_CALIBRATION_INKWELL)
				{
					calibrationStyle = HD_CALIBRATION_INKWELL;
				}
				if (supportedCalibrationStyles & HD_CALIBRATION_AUTO)
				{
					calibrationStyle = HD_CALIBRATION_AUTO;
				}

				//Calibrao
				if (calibrationStyle == HD_CALIBRATION_ENCODER_RESET)
				{
					hdUpdateCalibration(calibrationStyle);
					if (hdCheckCalibration() == HD_CALIBRATION_OK)
					{
						verify = 1;
					}
					if (HD_DEVICE_ERROR(error = hdGetError()))
					{
						verify = -3;//Erro na calibrao
					}           
				}
				*/
				
				
				/*if (calibrationStyle  == HD_CALIBRATION_INKWELL)
				{
					if (GetCalibrationStatus() == HD_CALIBRATION_NEEDS_MANUAL_INPUT)
					{
						printf("Please place the device into the inkwell ");
						printf("for calibration.\n\n");
					}
				}*/

				/////////////////////////////////////////
			
				verify = 1;

				//Zerar inicialmente os elementos da matriz de transformao
				for (int j = 0; j < 16; j++)
				{
					transform[j] = 0;
				}

		}
					
	}

	return verify;
}

//Encerra a comunicao com o dispositivo e finaliza o Scheduler
JNIEXPORT void JNICALL 
Java_ViMeT_NativeHaptic_finishDevice(JNIEnv *env, jobject obj)
{
	if (hHD != HD_INVALID_HANDLE)
	{
		delete position;
		delete velocity;
		delete transform;
		delete force;
		hdStopScheduler();
		hdUnschedule(hMain);
		hdDisableDevice(hHD);
	}
}

//Obtm a fora calculada pela aplicao Java
JNIEXPORT void JNICALL 
Java_ViMeT_NativeHaptic_sendData(JNIEnv *env, jobject obj, jdoubleArray dataarray)
{
	env->GetDoubleArrayRegion(dataarray, 0, 3, force);
	env->ReleaseDoubleArrayElements(dataarray,force,0);
}

//Retorna a matriz de transformao
JNIEXPORT jdoubleArray JNICALL 
Java_ViMeT_NativeHaptic_receiveData(JNIEnv *env, jobject obj)
{
		jdoubleArray array2 = env->NewDoubleArray(16);
		env->SetDoubleArrayRegion(array2,0,16,(jdouble*)transform);
		return array2;
}

//Retorna a posio do dispositivo nos eixos x, y e z
JNIEXPORT jdoubleArray JNICALL 
Java_ViMeT_NativeHaptic_receiveTranslation(JNIEnv *env, jobject obj)
{
	jdoubleArray array3 = env->NewDoubleArray(3);
	env->SetDoubleArrayRegion(array3,0,3,(jdouble*)position);
	return array3;
}

//Verifica se a CALLBACK continua rodando normalmente
JNIEXPORT jint JNICALL 
Java_ViMeT_NativeHaptic_verifyCallback(JNIEnv *env, jobject obj)
{
	if (!hdWaitForCompletion(hMain, HD_WAIT_CHECK_STATUS))
	{
		verifyThread = -1;
	}
	else
	{
		verifyThread = 0;
	}

	return verifyThread;
}

//Obtm a velocidade de movimentao do dispositivo
JNIEXPORT jdoubleArray JNICALL 
Java_ViMeT_NativeHaptic_receiveVelocity(JNIEnv *env, jobject obj)
{
	jdoubleArray array4 = env->NewDoubleArray(3);
	env->SetDoubleArrayRegion(array4,0,3,(jdouble*)velocity);
	return array4;
}

//CALLBACK principal
HDCallbackCode HDCALLBACK MainCallback(void *data)
{
	hdBeginFrame(hdGetCurrentDevice());

		//Funes de estado do OpenHaptic Toolkit

		//Funo para obter posio
		hdGetDoublev(HD_CURRENT_POSITION, position);

		//Funo para obter matriz de transformao
		hdGetDoublev(HD_CURRENT_TRANSFORM, transform);

		//Funo para obter a velocidade
		/*hdGetDoublev(HD_CURRENT_VELOCITY, velocity);*/

		//Funo para obter o estado do boto 1
		hdGetIntegerv(HD_CURRENT_BUTTONS, &currentButton1);
		hdGetIntegerv(HD_LAST_BUTTONS, &lastButton1);

		if ((currentButton1 & HD_DEVICE_BUTTON_1) != 0 &&
		(lastButton1 & HD_DEVICE_BUTTON_1) == 0)
		{
			button1 = 1;  //Boto 1 pressionado
		}
		else if ((currentButton1 & HD_DEVICE_BUTTON_1) == 0 &&
		(lastButton1 & HD_DEVICE_BUTTON_1) != 0)
		{
			button1 = 0; //Boto 1 est liberado
		}

		//Funo para obter o estado do Boto 2
		hdGetIntegerv(HD_CURRENT_BUTTONS, &currentButton2);
		hdGetIntegerv(HD_LAST_BUTTONS, &lastButton2);

		if ((currentButton2 & HD_DEVICE_BUTTON_2) != 0 &&
		(lastButton2 & HD_DEVICE_BUTTON_2) == 0)
		{
			button2 = 1; //Boto 2 est pressionado
		}
		else if ((currentButton2 & HD_DEVICE_BUTTON_2) == 0 &&
		(lastButton2 & HD_DEVICE_BUTTON_2) != 0)
		{
			button2 = 0;  //Boto 2 est liberado
		}
	
		//Realiza o retorno de fora
		//Funo para o retorno de fora
		hdSetDoublev(HD_CURRENT_FORCE, force);


	hdEndFrame(hdGetCurrentDevice());

	//Verifica erros durante a execuo da CALLBACK
	if (HD_DEVICE_ERROR(error = hdGetError()))
	{
			fprintf(stderr,"CALLBACK encerrada");
			getch();
			return HD_CALLBACK_DONE;
	}

	return HD_CALLBACK_CONTINUE;
}

//Callback para executar a calibrao
HDCallbackCode HDCALLBACK UpdateCalibrationCallback(int pUserData)
{
    int calibrationStyle = pUserData;

    if (hdCheckCalibration() == HD_CALIBRATION_NEEDS_UPDATE)
    {
        hdUpdateCalibration(calibrationStyle);
    }

    return HD_CALLBACK_DONE;
}
